/****************************** Module Header ******************************\
*
* Module Name: _mmf.c
*
* memory mapped files manager sample
*
* Copyright (c) WPS Toolkit Project - Christian Langanke 2000
*
* $Id: _mmf.c,v 1.2 2002/11/22 12:38:22 cla Exp $
*
* ===========================================================================*
*
* This file is part of the WPS Toolkit package and is free software.  You can
* redistribute it and/or modify it under the terms of the GNU Library General
* Public License as published by the Free Software Foundation, in version 2
* as it comes in the "COPYING.LIB" file of the WPS Toolkit main distribution.
* This library is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
* License for more details.
*
\***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define INCL_ERRORS
#define INCL_DOSMISC
#include <os2.h>

#define INCL_WTKMMF
#define INCL_WTKUTLFILE
#include <wtk.h>

// -----------------------------------------------------------------------------

#define PRINTSEPARATOR printf("\n------------------------------------------\n")

int main ( int argc, char *argv[])
{

         APIRET         rc = NO_ERROR;
         HMMF           hmmf = NULLHANDLE;
         CHAR           szFile[ _MAX_PATH];
         PSZ            pszMemory = NULL;
         PSZ            pszFileContents = NULL;
         ULONG          ulFilesize = 32 * MMF_MAXSIZE_KB;
         ULONG          ulCurrentSize;
         ULONG          ulBytesWritten;

static   PSZ            pszFile = "\?:\\CONFIG.SYS";

do
   {
   // --------------------------------------------------------------------------------

   PRINTSEPARATOR;

   // ----------  create in-memory only file --------------

   printf( "initialize\n");
   printf( "- WtkInitializeMmf: initialize support for memory mapped files\n");
   rc = WtkInitializeMmf( &hmmf, 4);
   if (rc != NO_ERROR)
      {
      printf( " WtkInitializeMmf: error: cannot initialize MMF support, rc=%u\n", rc);
      break;
      }

   // --------------------------------------------------------------------------------

   PRINTSEPARATOR;

   printf( "Testing in-memory pseudo file\n");

   // ----------  create in-memory only file --------------

   printf( "- WtkAllocMmf: allocate in-memory only buffer\n");
   rc = WtkAllocMmf( hmmf, (PVOID*) &pszMemory, MMF_FILE_INMEMORY, MMF_ACCESS_READWRITE, ulFilesize);
   if (rc != NO_ERROR)
      {
      printf( " WtkAllocMmf:  error: cannot allocate in-memory mapped file, rc=%u\n", rc);
      break;
      }

   // set memory to all hashmarks
   printf( "- access dynamic memory buffer, writing %u bytes\n", ulFilesize);
   memset( pszMemory, '#', ulFilesize);
   printf( "- no trap so far !\n");

   // --------------------------------------------------------------------------------

   PRINTSEPARATOR;

   printf( "Testing readwrite file access\n");

   // ----------  write dummy file --------------

   // allocate test file
   sprintf( szFile, "%s\\mmftest.txt", getenv( "TMP"));
   printf( "- WtkAllocMmf: allocate readwrite file buffer: %s\n", szFile);
   rc = WtkAllocMmf( hmmf, (PVOID*) &pszFileContents, szFile,
                  MMF_ACCESS_READWRITE | MMF_OPENMODE_RESETFILE,
                  ulFilesize);
   if (rc != NO_ERROR)
      {
      printf( " WtkAllocMmf: error: cannot allocate memory mapped file %s, rc=%u\n", szFile, rc);
      break;
      }

   // access memory by copying the contents of the in-memory file
   // but leave out first 64 bytes
   printf( "- access dynamic memory buffer of file, writing %u bytes\n", ulFilesize - 64);
   strcpy( pszFileContents, "This is the WPSTK testcase for memory mapped files !");
   memcpy( pszFileContents + 64, pszMemory, ulFilesize - 64);
   printf( "- no trap so far !\n");

   // cut size to half of size
   ulFilesize /= 2;
   printf( "- WtkSetMmfSize: cut size to %u\n", ulFilesize);
   rc = WtkSetMmfSize( hmmf, pszFileContents, ulFilesize);
   if (rc != NO_ERROR)
      {
      printf( " WtkSetMmfSize: error: cannot set size of memory buffer for file %s, rc=%u\n", szFile, rc);
      break;
      }

   // query size of memory area
   rc = WtkQueryMmfSize( hmmf, pszFileContents, &ulCurrentSize);
   if (rc != NO_ERROR)
      {
      printf( " WtkQueryMmfSize: error: cannot query size of allocated memory for memory mapped file %s, rc=%u\n", szFile, rc);
      break;
      }
   printf( "- WtkQueryMmfSize: file area size is %u\n", ulCurrentSize);

   // update the file
   printf( "- WtkUpdateMmf: update file\n");
   rc = WtkUpdateMmf( hmmf, pszFileContents);
   if (rc != NO_ERROR)
      {
      printf( " WtkUpdateMmf: error: cannot update: %s\n", szFile);
      break;
      }

   // query file size with file API
   printf( "- WtkQueryFileSize: checked size of updated file is %u\n", WtkQueryFileSize( szFile));

   // --------------------------------------------------------------------------------

   PRINTSEPARATOR;

   printf( "Release dynamic memory buffers used until here\n");


   // free memory and file again
   printf( "- WtkFreeMmf: free in-memory buffer\n");
   rc = WtkFreeMmf( hmmf, pszFileContents);
   if (rc != NO_ERROR)
      {
      printf( " WtkFreeMmf: error: cannot free memory for: %s\n", szFile);
      break;
      }

   // free memory or in-memory file
   printf( "- WtkFreeMmf: free file buffer\n");
   rc = WtkFreeMmf( hmmf, pszMemory);
   if (rc != NO_ERROR)
      {
      printf( " WtkFreeMmf: error: cannot free memory of in-memory file\n", szFile);
      break;
      }

   // --------------------------------------------------------------------------------

   PRINTSEPARATOR;

   printf( "Releaseing dynamic memory buffers used until here\n");

   // ----------  read config sys file --------------

   // open up config.sys
   printf( "- WtkAllocMmf: allocate readonly file: %s\n", pszFile);
   rc = WtkAllocMmf( hmmf, (PVOID*) &pszFileContents, pszFile, MMF_ACCESS_READONLY, 1024*1024);
   if (rc != NO_ERROR)
      {
      printf( " WtkAllocMmf: error: cannot allocate memory mapped file %s, rc=%u\n", pszFile, rc);
      break;
      }

   // display length of config.sys
   printf( "- strlen: contents of file is %u bytes long\n", strlen( pszFileContents));

   // free memory and file again
   printf( "- WtkFreeMmf: free file area\n");
   rc = WtkFreeMmf( hmmf, pszFileContents);
   if (rc != NO_ERROR)
      {
      printf( " WtkFreeMmf: error: cannot free memory for: %s\n", szFile);
      break;
      }

   // --------------------------------------------------------------------------------

   PRINTSEPARATOR;


   } while (FALSE);

// cleanup
printf( "cleanup\n");
if (hmmf)
   {
   printf( "- WtkTerminateMmf: terminate support for memory mapped files\n");
   WtkTerminateMmf( hmmf);
   }
printf( "\n");

return rc;
}

