/****************************** Module Header ******************************\
*
* Module Name: _nlstestinc.c
*
* NLS functions related sample include file
*
* ### NOTE: this file must be named *.c for checktest.cmd ###
*
* Copyright (c) WPS Toolkit Project - Christian Langanke 2000
*
* $Id: _nlstestinc.c,v 1.2 2004/04/15 21:27:51 cla Exp $
*
* ===========================================================================*
*
* This file is part of the WPS Toolkit package and is free software.  You can
* redistribute it and/or modify it under the terms of the GNU Library General
* Public License as published by the Free Software Foundation, in version 2
* as it comes in the "COPYING.LIB" file of the WPS Toolkit main distribution.
* This library is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
* License for more details.
*
\***************************************************************************/

#define DEFAULTLANGUAGE  "eng"
#define ENVVARLANGUAGE   "_NLS_LANGUAGE"
#define RESMASKPATH      "_nls%s.nls;dll\\_nls%s.nls"
#define INFMASKPATH      "_nls%s.inf;inf\\_nls%s.inf"

#define PRINTSEPARATOR printf("\n------------------------------------------\n")

#define CALLED_BY_EXE  TRUE
#define CALLED_BY_DLL  FALSE

// -----------------------------------------------------------------------------

static APIRET NlsExectuteTestcases( BOOL fExeCall)
{

         APIRET         rc = NO_ERROR;
         BOOL           fResult = FALSE;
         PSZ            pszDefaultLanguage;
         PSZ            pszEnvVarLanguage;
         PSZ            pszEnvValueLanguage;
         PSZ            pszResMaskPath;
         PSZ            pszInfMaskPath;

         HAB            hab = NULLHANDLE;
         CHAR           szFile[ _MAX_PATH];
         HMODULE        hmodRes = NULLHANDLE;
         CHAR           szString[ _MAX_PATH];

do
   {

   PRINTSEPARATOR;

   pszDefaultLanguage  = DEFAULTLANGUAGE;
   pszEnvVarLanguage   = ENVVARLANGUAGE;
   pszEnvValueLanguage = getenv( pszEnvVarLanguage);
   pszResMaskPath      = RESMASKPATH;
   pszInfMaskPath      = INFMASKPATH;

   printf( "\n");
   printf( "  using default langugage: %s\n", pszDefaultLanguage);
// printf( "          system language: %s\n", WtkQuerySysLanguage());
   printf( "             env variable: %s\n", pszEnvVarLanguage);
   printf( "    value of env variable: %s\n", (pszEnvValueLanguage) ? pszEnvValueLanguage : "<not set>");
   printf( "mask path for res modules: %s\n", pszResMaskPath);
   printf( "  mask path for INF files: %s\n", pszInfMaskPath);
   printf( "\n");

   // --------------------------------------------------------------------------------

   printf( "- determine language resource module filename\n");

   if (fExeCall)
      rc = WtkGetNlsPackageFilename( pszDefaultLanguage, pszEnvVarLanguage,
                                     pszResMaskPath, szFile, sizeof( szFile));
   else
      rc = WtkGetModNlsPackageFilename( (PFN)NlsExectuteTestcases, pszDefaultLanguage, pszEnvVarLanguage,
                                        pszResMaskPath, szFile, sizeof( szFile));

   if (rc != NO_ERROR)
      {
      printf( " WtkGet(Mod)NlsPackageFilename: error: cannot determine NLS package file. rc=%u\n", rc);
      break;
      }
   printf( "  - filename is: %s\n", szFile);

   // ------ ------ ------ ------ ------

   printf( "- load NLS specific resource file\n");

   if (fExeCall)
      rc = WtkLoadNlsResourceModule( &hmodRes, pszDefaultLanguage, pszEnvVarLanguage, pszResMaskPath);
   else
      rc = WtkLoadModNlsResourceModule( (PFN)NlsExectuteTestcases, &hmodRes, pszDefaultLanguage, pszEnvVarLanguage, pszResMaskPath);

   if (rc != NO_ERROR)
      {
      printf( " WtkLoad(Mod)NlsResourceModule: error: cannot load NLS specific resource file. rc=%u\n", rc);
      break;
      }
   printf( "  - resource module handle: %u\n", hmodRes);

   // ------ ------ ------ ------ ------

   printf( "- load string from resource file\n");
   hab = WinInitialize( 0);
   if (!WinLoadString( hab, hmodRes, IDSTR_LANGUAGE_IDENTIFIER, sizeof( szString), szString))
      {
      rc = ERRORIDERROR( WinGetLastError( hab));
      printf( "error: cannot load string from NLS resource file. rc=%u/0x%04x\n", rc, rc);
      break;
      }
   printf( "  - loaded string is: %s\n", szString);

   // ------ ------ ------ ------ ------

   printf( "- launch NLS specific INF file\n");

   if (fExeCall)
      rc = WtkLoadNlsInfFile( pszDefaultLanguage, pszEnvVarLanguage, pszInfMaskPath, "SAMPLEPAGE");
   else
      rc = WtkLoadModNlsInfFile( (PFN)NlsExectuteTestcases, pszDefaultLanguage, pszEnvVarLanguage, pszInfMaskPath, "SAMPLEPAGE");

   if (rc != NO_ERROR)
      {
      printf( " WtkLoad(Mod)NlsInfFile: error: cannot load NLS specific INF file. rc=%u\n", rc);
      break;
      }
   printf( "  - loaded INF file successfully\n");

   } while (FALSE);

// cleanup
if (hmodRes) DosFreeModule( hmodRes);
if (hab) WinTerminate( hab);
return rc;
}

