/****************************** Module Header ******************************\
*
* Module Name: _pm.c
*
* PM helper functions sample
*
* Copyright (c) WPS Toolkit Project - Christian Langanke 2004
*
* $Id: _pm.c,v 1.5 2004/04/19 19:14:50 cla Exp $
*
* ===========================================================================
*
* This file is part of the WPS Toolkit package and is free software.  You can
* redistribute it and/or modify it under the terms of the GNU Library General
* Public License as published by the Free Software Foundation, in version 2
* as it comes in the "COPYING.LIB" file of the WPS Toolkit main distribution.
* This library is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
* License for more details.
*
\***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <process.h>

#define INCL_ERRORS
#define INCL_DOS
#define INCL_WIN
#include <os2.h>

#define INCL_WTKUTLCONTROL
#define INCL_WTKUTLMODULE
#include <wtk.h>

#include <_pm.rch>

#define __TITLE__  "PM related helper functions sample"

#define WN_USER_INITSUBCLASS   (WM_USER + 0x1000)

#define TYPE_DEFNOTEBOOK         0
#define TYPE_OKNOTEBOOK          1

// -----------------------------------------------------------------------------

static VOID _initGuiControls( HWND hwnd, PVOID pvData)
{
         ULONG          ulWindowId =  WinQueryWindowUShort( hwnd, QWS_ID);

// setup controls
switch (ulWindowId)
   {
   case IDDLG_DEFNOTEBOOK_PAGE1:
      break;

   case IDDLG_DEFNOTEBOOK_PAGE2:
      break;

   case IDDLG_DEFNOTEBOOK_PAGE3:
      break;
   }

return;
}

// --------------------------------------------------------------------------

static VOID _controlGuiValues( HWND hwnd, PVOID pvData, MPARAM mp1, MPARAM mp2)
{
         ULONG          ulWindowId =  WinQueryWindowUShort( hwnd, QWS_ID);

// setup controls
switch (ulWindowId)
   {
   case IDDLG_DEFNOTEBOOK_PAGE1:
      break;

   case IDDLG_DEFNOTEBOOK_PAGE2:
      break;

   case IDDLG_DEFNOTEBOOK_PAGE3:
      break;
   }

return;
}

// --------------------------------------------------------------------------

static VOID _saveGuiValues( HWND hwnd, PVOID pvData)
{
         ULONG          ulWindowId =  WinQueryWindowUShort( hwnd, QWS_ID);


// save all values of GUI controls
switch (ulWindowId)
   {
   case IDDLG_DEFNOTEBOOK_PAGE1:
      break;

   case IDDLG_DEFNOTEBOOK_PAGE2:
      break;

   case IDDLG_DEFNOTEBOOK_PAGE3:
      break;
   }

return;
}

// --------------------------------------------------------------------------

static VOID _restoreGuiValues( HWND hwnd, PVOID pvData)
{
         ULONG          ulWindowId =  WinQueryWindowUShort( hwnd, QWS_ID);

// restore all values to GUI controls
switch (ulWindowId)
   {
   case IDDLG_DEFNOTEBOOK_PAGE1:
      break;

   case IDDLG_DEFNOTEBOOK_PAGE2:
      break;

   case IDDLG_DEFNOTEBOOK_PAGE3:
      break;
   }

return;
}

// -----------------------------------------------------------------------------

static VOID _defaultGuiValues( HWND hwnd, PVOID pvData)
{
         ULONG          ulWindowId =  WinQueryWindowUShort( hwnd, QWS_ID);

// reset controls
switch (ulWindowId)
   {
   case IDDLG_DEFNOTEBOOK_PAGE1:
      break;

   case IDDLG_DEFNOTEBOOK_PAGE2:
      break;

   case IDDLG_DEFNOTEBOOK_PAGE3:
      break;
   }

return;
}

// -----------------------------------------------------------------------------

static ULONG _launchNotebook( HWND hwnd, PSZ pszHelpFile, ULONG ulNotebookType)
{
         ULONG          ulResult = MBID_ERROR;
         ULONG          i;

         ULONG          ulFrameId;
         ULONG          ulPageCount;
         PULONG         paulPageId;

static   ULONG          aulDefNotebookPageId[] = { IDDLG_DEFNOTEBOOK_PAGE1,
                                                   IDDLG_DEFNOTEBOOK_PAGE2,
                                                   IDDLG_DEFNOTEBOOK_PAGE3};
static   ULONG          aulOkNotebookPageId[] =  { IDDLG_OKNOTEBOOK_PAGE1,
                                                   IDDLG_OKNOTEBOOK_PAGE2,
                                                   IDDLG_OKNOTEBOOK_PAGE3};

#define DEFNOTEBOOK_PAGECOUNT (sizeof( aulDefNotebookPageId) / sizeof( ULONG))
#define OKNOTEBOOK_PAGECOUNT (sizeof( aulOkNotebookPageId) / sizeof( ULONG))

         NOTEBOOKPAGEINFO anpi[ 16]; // make sure that this number is large enough !
         NOTEBOOKINFO   ni;

do
   {
   switch (ulNotebookType)
      {
      case TYPE_DEFNOTEBOOK:
         ulFrameId   = IDDLG_DEFNOTEBOOK;
         paulPageId  = aulDefNotebookPageId;
         ulPageCount = DEFNOTEBOOK_PAGECOUNT;
         break;

      case TYPE_OKNOTEBOOK:
         ulFrameId   = IDDLG_OKNOTEBOOK;
         paulPageId  = aulOkNotebookPageId;
         ulPageCount = OKNOTEBOOK_PAGECOUNT;
         break;
      }

   // prepare notebook page info
   memset( anpi, 0, sizeof( anpi));
   for (i = 0; i < ulPageCount; i++, paulPageId++)
      {
      anpi[ i].ulDlgResId        = *paulPageId;
      anpi[ i].usPageStyle       = BKA_MAJOR | BKA_AUTOPAGESIZE;
      anpi[ i].usPageOrder       = BKA_LAST;
      anpi[ i].pszTabText        = NULL;
      anpi[ i].pszMinorTabText   = NULL;
      anpi[ i].pszStatusText     = NULL;

      if (ulNotebookType == TYPE_OKNOTEBOOK)
         {
         anpi[ i].ulOkButtonId     = IDPBS_OK;
         anpi[ i].ulCancelButtonId = IDPBS_CANCEL;
         anpi[ i].ulFocusId        = IDPBS_CANCEL;
         }
      else
         anpi[ i].ulFocusId        = IDPBS_UNDO;

      anpi[ i].ulUndoButtonId    = IDPBS_UNDO;
      anpi[ i].ulDefaultButtonId = IDPBS_DEFAULT;
      anpi[ i].ulHelpButtonId    = IDPBS_HELP;
      }

   anpi[ 0].ulHelpPanel = IDPNL_NOTEBOOKPAGE1;
   anpi[ 1].ulHelpPanel = IDPNL_NOTEBOOKPAGE2;
   anpi[ 2].ulHelpPanel = IDPNL_NOTEBOOKPAGE3;

   // prepare notebook
   memset( &ni, 0, sizeof( ni));
   ni.hwndParent        = HWND_DESKTOP;
   ni.hwndOwner         = hwnd;
   ni.ulDlgResId        = ulFrameId;
   ni.ulNotebookResId   = IDNBK_NOTEBOOK;
   ni.ulSysIconId       = NULL;
   ni.hmodResource      = NULLHANDLE;
   ni.pszHelpTitle      = "Help";
   ni.pvData            = NULL;
   ni.ulNbPageCount     = ulPageCount;
   ni.panpi             = anpi;
   ni.pfnwcmdInitialize = _initGuiControls;
   ni.pfnwcntControl    = _controlGuiValues;
   ni.pfnwcmdSave       = _saveGuiValues;
   ni.pfnwcmdRestore    = _restoreGuiValues;
   ni.pfnwcmdDefault    = _defaultGuiValues;
   ni.pszHelpLibrary    = pszHelpFile;
   ni.fUndoAllPages     = FALSE;
   ni.fDefaultAllPages  = FALSE;

   ulResult = WtkNotebookDlg( &ni);

   } while (FALSE);

return ulResult;
}

// -----------------------------------------------------------------------------

static PFNWP _getWindowProcPtr( HWND hwnd)
{
         HAB            hab = WinQueryAnchorBlock( HWND_DESKTOP);
         CHAR           szClassName[ 20];
         CLASSINFO      ci;

WinQueryClassName( hwnd, sizeof( szClassName), szClassName);
WinQueryClassInfo(  ERRORIDERROR( WinGetLastError( hab)), szClassName, &ci);
return ci.pfnWindowProc;
}

// ---------------------------------------------------------------------

static MRESULT EXPENTRY _MleSubclassProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
         APIRET         rc = NO_ERROR;
         HAB            hab = WinQueryAnchorBlock( HWND_DESKTOP);
static   PFNWP          pfnwpOrgWindowProc = NULL;
static   HWND           hwndMenu = NULLHANDLE;
static   CHAR           szHelpFile[ _MAX_PATH];

if (!pfnwpOrgWindowProc)
   pfnwpOrgWindowProc = _getWindowProcPtr( hwnd);

switch (msg)
   {
   case WN_USER_INITSUBCLASS:
      {
               ULONG          ulStyle;
               ULONG          ulMleWindowId =  WinQueryWindowUShort( hwnd, QWS_ID);

      // determine helpfile and create help instance
      rc = WtkGetModulePath( (PFN)_MleSubclassProc, szHelpFile, sizeof( szHelpFile));
      if (rc == NO_ERROR)
         strcat( szHelpFile, "\\_pm.hlp");
      WtkCreateHelpInstance( hwnd, "Help", szHelpFile);

      // load basic menu
      hwndMenu = WinLoadMenu( HWND_OBJECT, NULLHANDLE, IDMEN_POPUP);

      // demo usage of WtkInsertMenu
      // insert one menu into existing one
      // (this is helpful when extending the menu of XCenter widgets)
      WtkInsertMenu( hwnd, hwndMenu, IDMEN_OKNOTEBOOK, NULLHANDLE, IDMEN_INSERT, 
                     WTK_INSERTMENU_TOPSEPARATOR | WTK_INSERTMENU_BOTTOMSEPARATOR);

      // set style of MLE and change font
      WinSendMsg( hwnd, MLM_SETREADONLY, MPFROMLONG( TRUE), 0);
      WinSendMsg( hwnd, MLM_SETSEL, MPFROMLONG( 5), MPFROMLONG( 5));
      WinSendMsg( hwnd, MLM_SETWRAP, MPFROMLONG( TRUE), 0);
      WtkSetWindowFont( hwnd, "9.WarpSans Bold", 0);

      // set MLE content
      WtkAddTextResourceToMLE( WinQueryWindow( hwnd, QW_PARENT), 
                               ulMleWindowId, NULL, RT_USER_DATAFILE, IDRES_MLETEXT);
      }
      break;

   // -------------------------------------------------------

   case WM_BUTTON2UP:
      {
               POINTL         ptl;          
               ULONG          ulPopupMenuStyle = PU_HCONSTRAIN     | PU_VCONSTRAIN   |
                                                 PU_MOUSEBUTTON1   | PU_MOUSEBUTTON2 |
                                                 PU_MOUSEBUTTON3   | PU_KEYBOARD;
      // popup system menu
      WinQueryPointerPos( HWND_DESKTOP, &ptl);
      WinPopupMenu( HWND_DESKTOP,
                    hwnd,
                    hwndMenu,
                    ptl.x,
                    ptl.y,
                    0,
                    ulPopupMenuStyle);
      }
      break;

   // -------------------------------------------------------

   case WM_COMMAND:
      switch (LONGFROMMP( mp1))
         {
         case IDMEN_DEFNOTEBOOK:
            _launchNotebook( hwnd, szHelpFile, TYPE_DEFNOTEBOOK);
            break;

         case IDMEN_OKNOTEBOOK:
            _launchNotebook( hwnd, szHelpFile, TYPE_OKNOTEBOOK);
            break;

         case IDMEN_EXIT:
            WinPostMsg( hwnd, WM_QUIT, 0, 0);
            break;

         case IDMEN_ITEM1:
         case IDMEN_ITEM2:
         case IDMEN_ITEM3:
            {
                     CHAR           szMessage[ 127];
            sprintf( szMessage, "Inserted text menu item %u was selected!",
                     LONGFROMMP( mp1) - IDMEN_ITEM1 + 1);
            WinMessageBox( HWND_DESKTOP, hwnd, szMessage, 
                           __TITLE__, -1, MB_MOVEABLE | MB_OK | MB_INFORMATION);
            }
            break;

         case IDMEN_ITEMBITMAP:
            WinMessageBox( HWND_DESKTOP, hwnd, "Inserted bitmap menu item was selected!", 
                           __TITLE__, -1, MB_MOVEABLE | MB_OK | MB_INFORMATION);
            break;

         }
      return (MRESULT) TRUE;
      break;

   // -------------------------------------------------------

   case WM_HELP:
      WtkDisplayHelpPanel( hwnd, IDPNL_MAINWINDOW);
      return (MRESULT) TRUE;
      break;

   // -------------------------------------------------------

   case WM_DESTROY:
      if (hwndMenu)
         WinDestroyWindow( hwndMenu);
      WtkDestroyHelpInstance( hwnd);
      break;

   } // switch (msg)

return pfnwpOrgWindowProc( hwnd, msg, mp1, mp2);

}

// -----------------------------------------------------------------------------

int main ( int argc, char *argv[])
{

         APIRET         rc = NO_ERROR;
         BOOL           fResult = FALSE;

         HAB            hab = NULLHANDLE;
         HMQ            hmq = NULLHANDLE;
         QMSG           qmsg;

         HWND           hwndFrame;
         
         HWND           hwndMle;
         ULONG          ulFrameStyles = FCF_SIZEBORDER | FCF_TITLEBAR | FCF_SYSMENU |
                                        FCF_MINBUTTON | FCF_TASKLIST;
         ULONG          ulMLEStyle = 0;
do
   {
   // get PM resources
   if ((hab = WinInitialize( 0)) == NULLHANDLE)
      break;
   if ((hmq = WinCreateMsgQueue( hab, 0)) == NULLHANDLE)
      break;

   // create a standard window with MLE as client
   // so we don't have to deal with drawing issues for this sample
   hwndFrame = WinCreateStdWindow( HWND_DESKTOP, WS_VISIBLE, &ulFrameStyles, 
                                   WC_MLE, __TITLE__, ulMLEStyle,
                                   0, 0xffff,
                                   &hwndMle);
   if (!hwndFrame)
      {
      rc = ERRORIDERROR( WinGetLastError( hab));
      break;
      }

   // set window postition
   WinSetWindowPos( hwndFrame, HWND_TOP, 200, 200, 500, 500,
                    SWP_MOVE | SWP_SIZE | SWP_SHOW | SWP_ZORDER | SWP_ACTIVATE);

   // subclass MLE window to get further control
   WinSubclassWindow( hwndMle, _MleSubclassProc);

   // run initialization code of our subclass procedure
   WinPostMsg( hwndMle, WN_USER_INITSUBCLASS, 0, 0);

   // now dispatch messages
   while (WinGetMsg( hab, &qmsg, NULLHANDLE, 0, 0))
      {
      WinDispatchMsg(hab, &qmsg);
      }

   } while (FALSE);

// cleanup
if (hmq) WinDestroyMsgQueue( hmq);
if (hab) WinTerminate( hab);

return rc;
}
  
